@@ -58,6 +58,12 @@
 #'
 NULL
 
+#' @importFrom graphics legend par plot
+#' @importFrom stats as.formula glm na.exclude rbinom reshape rnorm runif setNames terms.formula
+#' @importFrom utils head str
+NULL
+
+
 # \item {\code{node}} - defines a node in the structural equation model and
 # its conditional distribution, i.e., the outcome of one equation in the
 # structural equation model and the formula that links the outcome value
